#include <stdio.h>

#include "Config.h"
#include "debugio.h"
#include "ReplacementList.h"

static const char configfile[] = "nAPI.conf";

typedef map<const string, func_opt> all_func_opts;
static all_func_opts monitor_options;

static bool find_function(const char *func) {
	for (const DLL_functions *dll = za_zamjenu; dll->dll_name; ++dll)
		for (const FuncPair *fd = dll->functions; fd->origf; ++fd)
			if (!strcmp(fd->origf, func))
				return true;
	return false;
}

/*
   potrazi da li je podrzavamo tu funkciju i dodatno pokusa odrediti da li se radi o A ili W funkciji.
   zc = AW znak
*/
static bool check_function(char *func, char **zc) {
	*zc = NULL;

	if (find_function(func))
		return true;

	*zc = func + strlen(func);
	
	*(*zc + 1) = 0;
	**zc = 'A';
	if (find_function(func)) {	// ima li Fja+A?
		**zc = 'W';
		if (!find_function(func)) {	// i da li ima Fja+W?
			**zc = 'A';		// ako nema, postoji samo +A
			*zc = NULL;
		}
		return true;
	}

	// nema Fja+A:
	**zc = 'W';
	if (find_function(func)) {	// ima li samo Fja+W?
		*zc = NULL;		// (jedina je)
		return true;
	}
	**zc = 0;
	*zc = NULL;
	return false;
}

bool LoadConfig() {
	int n;
	FILE *f;
       
	f = fopen(configfile, "rt");
	if (!f) return false;

	while (fscanf(f, "%d", &n) == 1) {
		char func[100], *zc;
		fscanf(f, "%s", func);

		// da li postoji?
		if (!check_function(func, &zc)) {
			wprintf("Nepodrzana funkcija: \"%s\"\n", func);
		}

		for (int i = 0; i < n; ++i) {
			char option[100];
			fscanf(f, "%s", option);

			if (!zc) {
				// oznaka da treba monitorati + podaci
				monitor_options[func][option] = true;
			} else {
				// samo oznaka da treba monitorati
				monitor_options[func]["ok"] = true;
				*zc = 'A';
				monitor_options[func]["ok"] = true;

				// podaci za logera ne smiju imat A ili W
				*zc = 0;
				monitor_options[func][option] = true;
			}
		}
	}

	fclose(f);
	return true;
}

bool FunctionMonitoringEnabled(const string &funcname) {
	return OptionExists<func_opt>(monitor_options, funcname);
}

const func_opt WhatToMonitorFor(const string &funcname) {
	all_func_opts::const_iterator ci = monitor_options.find(funcname);
	if (ci == monitor_options.end())
		return func_opt();
	return ci->second;
}
