#define _WIN32_WINNT   0x0400
#include <windows.h>
#include "FileAPIOverride.h"
#include "debugio.h"
#include "Locker.h"
#include "wcconv.h"
#include "Log.h"
#include <map>
#include "string.h"

#define MAXPATH 260

using namespace std;

typedef map<HANDLE,std::string> HandleList;

// klasa sa globalnim podacima za podatkovne apije
static class nAPIFileData : private Locker {
	public:
		nAPIFileData() {
			tr=0;
			tw=0;
			sum_read=0;
			sum_write=0;
		}
		
		// vraca sumu procitanih bajta
		unsigned int GetSumRead() {
			return sum_read;
		}

		// vraca sumu upisanih bajta
		unsigned int GetSumWrite() {
			return sum_write;
		}

		// vraca broj poziva ReadFile
		unsigned int GetNumRead() {
			return tr;
		}

		// vraca broj poziva WriteFile
		unsigned int GetNumWrite() {
			return tw;
		}
		
		//promjena globalnih podataka za citanje
		void ReadData(unsigned int nNumberOfBytesToRead){
			Lock();
			tr++;
			sum_read+=nNumberOfBytesToRead;
			Unlock();
		}
		
		//promjena globalnih podataka za pisanje
		void WriteData(unsigned int nNumberOfBytesToWrite){
			Lock();
			++tw;
			sum_write+=nNumberOfBytesToWrite;
			Unlock();
		}
		
		//upisuje handle u listu
		void NewHandle(HANDLE h, const std::string &filename) {
			Lock();
			hlst.insert(pair<HANDLE,std::string>(h,filename));
			Unlock();
		}
		//pronalazi i uklanja handle iz liste
		bool RemoveHandle(HANDLE h) {
			bool rh = false;
			Lock();
			HandleList::iterator i = hlst.find(h);
			if (i != hlst.end()) {
				hlst.erase(i);
				rh = true;
			}
			Unlock();
			return rh;
		}
		// trazi handle u listi
		bool HandleExists(HANDLE h) {
			bool a;
			Lock();
			a = hlst.find(h) != hlst.end();
			Unlock();
			return a;
		}
		//pretrazuje listu handleova i vraca ime datoteke
		std::string GetFileName(HANDLE h){
			std::string name;
			Lock();
			HandleList::iterator i = hlst.find(h);
			if (i != hlst.end()){
				name=i->second;
			}
			Unlock();	
			return name;
		}

	private:
		unsigned int sum_read;
		unsigned int sum_write;
		unsigned int tr;                   
		unsigned int tw;				   
		HandleList hlst;
} nmd;

static void FillDefaults(Report *r) {
	unsigned int tr,tw,sum_read,sum_write;
	sum_read = nmd.GetSumRead();
	tr = nmd.GetNumRead();
	sum_write = nmd.GetSumWrite();
	tw = nmd.GetNumWrite();
	r->AddInt("sum_read", sum_read);					//ukupno procitano bajta
	r->AddInt("num_read", tr);							//broj poziva funkcije ReadFile
	r->AddInt("avg_read", tr ? sum_read/tr : 0);		//prosjek procitanih podataka
	r->AddInt("sum_write", sum_write);					//ukupno zapisano bajta
	r->AddInt("num_write", tw);							//broj pozifa funkcije WriteFile
	r->AddInt("avg_written", tw ? sum_write/tw : 0);	//prosjek upisanih podataka
}

//funkcija za ispis gresaka
void GetErr(char *buffer) {
	int error;

	error = GetLastError();

	switch(error) {
		case ERROR_ALREADY_EXISTS:
			sprintf(buffer, "Datoteka/mapa vec postoji.");
			break;
		case (DWORD)-1:	// INVALID_HANDLE_VALUE
			sprintf(buffer, "Greska pri izvrsavanju funkcije");
			break;
		case ERROR_ACCESS_DENIED:
			sprintf(buffer, "Odbijen pristup datoteci.");
			break;
		case ERROR_FILE_NOT_FOUND:
			sprintf(buffer, "Datoteka nije pronadjena ili ne postoji.");
			break;
		case ERROR_HANDLE_EOF:
			sprintf(buffer, "Greska pri pokusaju citanja.(EOF)");
			break;
		case ERROR_IO_PENDING:
			sprintf(buffer, "Funkcija zavrsila prije kraja pisanja u datoteku.");
			break;
		case ERROR_REQUEST_ABORTED:
			sprintf(buffer, "Operacija kopiranja datoteke prekinuta.");
			break;
		case ERROR_BROKEN_PIPE:
			sprintf(buffer, "Greska kod otvaranja cjevovoda.");
			break;
		case ERROR_PATH_NOT_FOUND:
			sprintf(buffer, "Mapa nije pronadjena ili put nije ispravno zadan.");
			break;
		case 0:
			sprintf(buffer, "Nepoznata greska.");
			break;
		default:
			sprintf(buffer, "Greska: %d.", error);
	}

}


//kreiranje/otvaranje datoteke
static void nAPI_CreateFile(LPCSTR lpFileName, DWORD dwDesiredAccess, DWORD dwShareMode, LPSECURITY_ATTRIBUTES lpSecurityAttributes, DWORD dwCreationDistribution, DWORD dwFlagsAndAttributes, HANDLE hTemplateFile, HANDLE result) {
	Report r("CreateFile");
	char buffer[100];
	GetErr(buffer);
	nmd.NewHandle(result, lpFileName);
	dprintf("CreateFile(%s,0x%08X,0x%08X,%p,0x%08X,0x%08X,%p) =%p\n", lpFileName, (unsigned int)dwDesiredAccess,(unsigned int)dwShareMode,lpSecurityAttributes, (unsigned int)dwCreationDistribution,(unsigned int)dwFlagsAndAttributes,hTemplateFile,result);
	
	if (result==INVALID_HANDLE_VALUE){
		r.AddString("greska", buffer);
	} else{
		r.AddString("greska", "0");
	}
	r.AddString("file_name", lpFileName);
	r.AddHex("desired_access", dwDesiredAccess);
	r.AddHex("flags_attributes", dwFlagsAndAttributes);
	r.AddPtr("htemplatefile", hTemplateFile);	
	r.AddPtr("handle", result);
	
	FillDefaults(&r);
	AddToLog(r);
}

HANDLE __stdcall nAPI_CreateFileA(LPCSTR lpFileName, DWORD dwDesiredAccess, DWORD dwShareMode, LPSECURITY_ATTRIBUTES lpSecurityAttributes, DWORD dwCreationDistribution, DWORD dwFlagsAndAttributes, HANDLE hTemplateFile) {
	HANDLE result = CreateFileA(lpFileName,dwDesiredAccess,dwShareMode,lpSecurityAttributes,dwCreationDistribution,dwFlagsAndAttributes,hTemplateFile);
	nAPI_CreateFile(lpFileName,dwDesiredAccess,dwShareMode,lpSecurityAttributes,dwCreationDistribution,dwFlagsAndAttributes,hTemplateFile,result);
	return result;
}

HANDLE __stdcall nAPI_CreateFileW(LPCWSTR lpFileName, DWORD dwDesiredAccess, DWORD dwShareMode, LPSECURITY_ATTRIBUTES lpSecurityAttributes, DWORD dwCreationDistribution, DWORD dwFlagsAndAttributes, HANDLE hTemplateFile) {
	HANDLE result = CreateFileW(lpFileName,dwDesiredAccess,dwShareMode,lpSecurityAttributes,dwCreationDistribution,dwFlagsAndAttributes,hTemplateFile);
	nAPI_CreateFile(wcconvs(lpFileName).c_str(),dwDesiredAccess,dwShareMode,lpSecurityAttributes,dwCreationDistribution,dwFlagsAndAttributes,hTemplateFile,result);
	return result;
}

//brisanje datoteke
// datoteku nije moguce obrisati ukoliko lpFileName nije ispravan ili je handle na datoteku jos otvoren
// prilikom testiranja potrebno prvo izvrsiti CloseHandle ili datoteka nece biti obrisana
static void nAPI_DeleteFile(LPCSTR lpFileName, BOOL result){
	Report r("DeleteFile");
	char filename[MAXPATH];
	char buffer[100];
	strcpy(filename, lpFileName);
	GetErr(buffer);
	dprintf("DeleteFile(%s = %d \n )",lpFileName,(int)result);
	
	if(!result){
		printf("Pokusaj brisanja nepostojece ili otvorene datoteke!");
		r.AddString("greska", buffer);
	}
	else{
		r.AddString("greska", "0");
	}
	r.AddString("file_name",filename);
	r.AddInt("success",result);

	FillDefaults(&r);
	AddToLog(r);
}

BOOL __stdcall nAPI_DeleteFileA(LPCSTR lpFileName){
	BOOL result= DeleteFileA(lpFileName);
	nAPI_DeleteFile(lpFileName, result);
	return result;
}

BOOL __stdcall nAPI_DeleteFileW(LPCWSTR lpFileName){
	BOOL result= DeleteFileW(lpFileName);
	nAPI_DeleteFile(wcconvs(lpFileName).c_str(), result);
	return result;
}

//citanje podataka iz datoteke
BOOL __stdcall nAPI_ReadFile(HANDLE hFile,LPVOID lpBuffer,DWORD nNumberOfBytesToRead,LPDWORD lpNumberOfBytesRead,LPOVERLAPPED lpOverlapped){
	if (!nmd.HandleExists(hFile)) {           //za pokusaj citanja iz zatvorene datoteke/datoteke ciji handle nije u listi handleova
		dprintf("Handle %p ne postoji u listi\n", hFile);
		return FALSE;
	}
	Report r("ReadFile");
	BOOL result;
	char buffer[100];
	GetErr(buffer);	
	std::string name = nmd.GetFileName(hFile);
	const char* ime=name.c_str(); //castanje stringa u const char*
	result = ReadFile(hFile,lpBuffer,nNumberOfBytesToRead,lpNumberOfBytesRead,lpOverlapped);
	dprintf("ReadFile(%p,%p,0x%08X,%p,%p) = %d \n", hFile,lpBuffer,(unsigned int)nNumberOfBytesToRead,lpNumberOfBytesRead, lpOverlapped,(int)result);

	if (result) {	// ako je uspjesno procitano obnovi globalne podatke
		nmd.ReadData((unsigned int)nNumberOfBytesToRead); 
		r.AddString("greska","0");
	}
	else{
		r.AddString("greska",buffer);
	}
	r.AddPtr("handle", hFile);
	r.AddString("file_name", ime);
	r.AddHex("bytes_read", nNumberOfBytesToRead);
	r.AddPtr("read_address", lpNumberOfBytesRead);
	r.AddInt("success", result);
	
	FillDefaults(&r);
	AddToLog(r);

	return result;
}

//pisanje podataka u datoteku
BOOL __stdcall nAPI_WriteFile(HANDLE hFile,LPCVOID lpBuffer,DWORD nNumberOfBytesToWrite,LPDWORD lpNumberOfBytesWritten,LPOVERLAPPED lpOverlapped){
	if (!nmd.HandleExists(hFile)) {					//provjera za pokusaj pisanja u zatvorenu datoteku/datoteku ciji handle nije zapisan u listu
		dprintf("Handle %p ne postoji u listi\n", hFile);
		return FALSE;
	}
	Report r("WriteFile");
	BOOL result;
	char buffer[100];
	GetErr(buffer);
	std::string name = nmd.GetFileName(hFile);
	const char* ime=name.c_str();    //castanje stringa u const char*
	result= WriteFile(hFile,lpBuffer,nNumberOfBytesToWrite,lpNumberOfBytesWritten,lpOverlapped);
	dprintf("WriteFile(%p,%p,0x%08X,%p,%p,= %d\n",hFile,lpBuffer,(unsigned int)nNumberOfBytesToWrite,lpNumberOfBytesWritten,lpOverlapped,(int)result);

	if (result) {	// ako je uspjesno zapisano
		nmd.WriteData((unsigned int)nNumberOfBytesToWrite); //obnovi globalne podatke
		r.AddString("greska", "0");
	}
	else{
		r.AddString("greska", buffer);
	}
	r.AddPtr("handle", hFile);
	r.AddString("file_name", ime);	
	r.AddHex("bytes_write", nNumberOfBytesToWrite);
	r.AddPtr("write_address", lpNumberOfBytesWritten);
	r.AddInt("success", result);

	FillDefaults(&r);
	AddToLog(r);

	return result;
}

//premjestanje datoteke
static void nAPI_MoveFile(LPCSTR lpExistingFileName,LPCSTR lpNewFileName, BOOL result){
	Report r("MoveFile");
	char buffer[100];
	GetErr(buffer);
	//pokusaj premjestanja otvorene (prije CloseHandle) ili nepostojece datoteke vraca vrijednost success=0;
	dprintf("MoveFile(%s, %s = %d \n)",lpExistingFileName,lpNewFileName,(int)result);
	
	if(!result){
		r.AddString("greska", buffer);	
	}
	else{
		r.AddString("greska", "0");	
	}
	r.AddString("oldfilename", lpExistingFileName);
	r.AddString("newfilename", lpNewFileName);
	r.AddInt("success", result);
	
	FillDefaults(&r);
	AddToLog(r);
}

BOOL __stdcall nAPI_MoveFileA(LPCSTR lpExistingFileName,LPCSTR lpNewFileName){
	BOOL result = MoveFileA(lpExistingFileName,lpNewFileName);
	nAPI_MoveFile(lpExistingFileName,lpNewFileName, result);
	return result;
}

BOOL __stdcall nAPI_MoveFileW(LPCWSTR lpExistingFileName,LPCWSTR lpNewFileName){
	BOOL result = MoveFileW(lpExistingFileName,lpNewFileName);
	nAPI_MoveFile( wcconvs(lpExistingFileName).c_str(), wcconvs(lpNewFileName).c_str(),result);
	return result;
}
//kopiranje datoteke
static void nAPI_CopyFileEx(LPCSTR lpExistingFileName,LPCSTR lpNewFileName,LPPROGRESS_ROUTINE lpProgressRoutine,LPVOID lpData,LPBOOL pbCancel,DWORD dwCopyFlags,BOOL result){
	Report r("CopyFileEx");
	char buffer[100];
	GetErr(buffer);
	dprintf("CopyFileEx(%s, %s,%p,%p,%p,0x%08X = %d",lpExistingFileName,lpNewFileName,lpProgressRoutine,lpData,pbCancel,(unsigned int)dwCopyFlags,(int)result);

	if(!result){
		r.AddString("greska", buffer);
	}
	else{
		r.AddString("greska", "0");
	}
	r.AddString("originalfilename", lpExistingFileName);
	r.AddString("newfilename", lpNewFileName);
	r.AddHex("flags", dwCopyFlags);
	r.AddInt("success", result);

	FillDefaults(&r);
	AddToLog(r);
}

BOOL __stdcall nAPI_CopyFileExA(LPCSTR lpExistingFileName,LPCSTR lpNewFileName,LPPROGRESS_ROUTINE lpProgressRoutine,LPVOID lpData,LPBOOL pbCancel,DWORD dwCopyFlags){
	BOOL result = CopyFileExA(lpExistingFileName,lpNewFileName,lpProgressRoutine,lpData,pbCancel,dwCopyFlags);
	nAPI_CopyFileEx(lpExistingFileName,lpNewFileName,lpProgressRoutine,lpData,pbCancel,dwCopyFlags,result);
	return result;
}

BOOL __stdcall nAPI_CopyFileExW(LPCWSTR lpExistingFileName,LPCWSTR lpNewFileName,LPPROGRESS_ROUTINE lpProgressRoutine,LPVOID lpData,LPBOOL pbCancel,DWORD dwCopyFlags){
	BOOL result = CopyFileExW(lpExistingFileName,lpNewFileName,lpProgressRoutine,lpData,pbCancel,dwCopyFlags);
	nAPI_CopyFileEx(wcconvs(lpExistingFileName).c_str(),wcconvs(lpNewFileName).c_str(),lpProgressRoutine,lpData,pbCancel,dwCopyFlags,result);
	return result;
}
//kreiranje nove mape
static void nAPI_CreateDirectory(LPCSTR lpPathName,LPSECURITY_ATTRIBUTES lpSecurityAttributes, BOOL result){
	Report r("CreateDirectory");
	char buffer[100];
	GetErr(buffer);
	dprintf("CreateDirectory(%s, %p = %d \n)", lpPathName, lpSecurityAttributes, (int)result);
	
	if(!result){
		r.AddString("greska", buffer);
	}
	else{
		r.AddString("greska", "0");
	}
	r.AddString("path_name", lpPathName);
	r.AddPtr("attributes", lpSecurityAttributes);
	r.AddInt("success", result);

	FillDefaults(&r);
	AddToLog(r);
}

BOOL __stdcall nAPI_CreateDirectoryA(LPCSTR lpPathName,LPSECURITY_ATTRIBUTES lpSecurityAttributes){
	BOOL result = CreateDirectoryA(lpPathName, lpSecurityAttributes);
	nAPI_CreateDirectory(lpPathName,lpSecurityAttributes,result);
	return result;
}

BOOL __stdcall nAPI_CreateDirectoryW(LPCWSTR lpPathName,LPSECURITY_ATTRIBUTES lpSecurityAttributes){
	BOOL result = CreateDirectoryW(lpPathName,lpSecurityAttributes);
	nAPI_CreateDirectory(wcconvs(lpPathName).c_str(),lpSecurityAttributes,result);
	return result;
}
//brisanje mape
static void nAPI_RemoveDirectory(LPCSTR lpPathName, BOOL result){
	Report r("RemoveDirectory");
	char buffer[100];
	GetErr(buffer);
	dprintf("RemoveDirectory(%s = %d \n)", lpPathName, (int)result);
	
	if(!result){
		r.AddString("greska", buffer);
	}
	else{
		r.AddString("greska", "0");
	}
	r.AddString("path_name", lpPathName);
	r.AddInt("success", result);

	FillDefaults(&r);
	AddToLog(r);
}

BOOL __stdcall nAPI_RemoveDirectoryA(LPCSTR lpPathName){
	BOOL result = RemoveDirectoryA(lpPathName);
	nAPI_RemoveDirectory(lpPathName,result);
	return result;
}

BOOL __stdcall nAPI_RemoveDirectoryW(LPCWSTR lpPathName){
	BOOL result = RemoveDirectoryW(lpPathName);
	nAPI_RemoveDirectory(wcconvs(lpPathName).c_str(),result);
	return result;
}

//zatvaranje handlea na datoteku
BOOL __stdcall nAPI_CloseHandleFile(HANDLE hObject){
	if (!nmd.HandleExists(hObject)) {            //provjeri postoji li handle u listi
		dprintf("%p ne postoji u listi\n", hObject);
		return FALSE;
	}
	Report r("CloseHandle");
	BOOL result;
	char buffer[100];
	bool success = false;
	result = CloseHandle(hObject);
	dprintf("CloseHandle(%p = %d)\n", hObject, (int)result);
	if (result){
		success = nmd.RemoveHandle(hObject);
		r.AddString("greska","0");
	}
	else{
		r.AddString("greska",buffer);
	}
	r.AddString("type", "FILE_HANDLE");
	r.AddPtr("handle", hObject);
	r.AddInt("success", result);

	FillDefaults(&r);
	AddToLog(r);

	return result;
}
