#include <windows.h>
#include <stdio.h>
#include <string.h>

#include "Log.h"
#include "Config.h"

static const char logfilenameprefix[] = "nAPI-";
static const char logfilenameextension[] = ".txt";

static FILE *log;
static CRITICAL_SECTION logsection;

bool InitLog() {
	char mfn[256], *t;
	string lfn = logfilenameprefix;

	GetModuleFileName(NULL, mfn, sizeof(mfn));
	t = strrchr(mfn, '/');
	if (t) ++t; else t = mfn;
	t = strrchr(t, '\\');
	if (t) ++t; else t = mfn;
	lfn += t;
	lfn += logfilenameextension;

	log = fopen(lfn.c_str(), "wt");
	if (!log) {
		return false;
	}

	InitializeCriticalSection(&logsection);
	return true;
}

void CloseLog() {
	fclose(log);
	DeleteCriticalSection(&logsection);
}

void AddToLog(const Report &r) {
	EnterCriticalSection(&logsection);
	r.WriteToFile(log);
	LeaveCriticalSection(&logsection);
}




void Report::AddElement(const ReportElement &re) {
	elems.push_back(re);
}

void Report::AddString(const string &name, const char *str) {
	AddElement(ReportElement(name, FormatString(str)));
}

void Report::AddString(const string &name, const string &str) {
	AddElement(ReportElement(name, FormatString(str.c_str())));
}

void Report::AddHex(const string &name, unsigned int hex) {
	char buf[16];
	sprintf(buf, "0x%08X", hex);
	AddElement(ReportElement(name, buf));
}

void Report::AddPtr(const string &name, void *ptr) {
	char buf[64];
	if (ptr) {
		sprintf(buf, "0x%p", ptr);
	} else {
		strcpy(buf, "(null)");
	}
	AddElement(ReportElement(name, buf));
}

void Report::AddInt(const string &name, int broj) {
	char buf[20];
	sprintf(buf, "%d", broj);
	AddElement(ReportElement(name, buf));
}

void Report::AddUInt(const string &name, UINT broj) {
	char buf[20];
	sprintf(buf, "%u", broj);
	AddElement(ReportElement(name, buf));
}

void Report::AddDouble(const string &name, double broj) {
	char buf[25];
	sprintf(buf, "%lf", broj);
	AddElement(ReportElement(name, buf));
}

void Report::WriteToFile(FILE *fjl) const {
	const func_opt fopt = WhatToMonitorFor(fname);
	vector<ReportElement>::const_iterator ci;
	string tmp;
	int n = 0;

	bool monitor_all = OptionExists<bool>(fopt, "/all/");

	for (ci = elems.begin(); ci != elems.end(); ++ci) {
		if (monitor_all || OptionExists<bool>(fopt, ci->name.c_str())) {
			tmp += ci->name + ": " + ci->value + "\n";
			++n;
		}
	}
	fprintf(fjl, "%d %s\n%s\n", n, fname.c_str(), tmp.c_str());
}

string Report::FormatString(const char *s) {
	if (!s) return "(null)";

	char tmp[20];
	const char *t;
	int nlines;

	for (nlines = 1, t = s; *t; ++t)
		nlines += *t == '\n';
	sprintf( tmp, "\" %d\n", nlines);

	return string(tmp) + s + "\n\"" ;
}
