#include <windows.h>
#include "MemoryAPIOverride.h"
#include "ErrorDescriber.h"
#include "debugio.h"
#include "Locker.h"
#include "Log.h"

#define NUM_BLOKOVA 100

// klasa sa globalnim podacima za memory apije
static class nAPIMemoryData : private Locker {
	public:
		// konstruktor
		nAPIMemoryData() {
			// inicijalizira podatke
			sum = 0;
			num = 0;
			ZeroMemory(blok, sizeof(blok));
		}

		// vraca sumu alociranih blokova
		unsigned int GetSum() {
			return sum;
		}

		// vraca broj alociranih blokova
		unsigned int GetNum() {
			return num;
		}

		// doda novi blok
		void AddData(HGLOBAL handle, unsigned int size) {
			Lock();		// udje u kriticni odsjecak

			// trazi prvi slobodan blok:
			for (int i = 0; i < NUM_BLOKOVA; ++i) {
				if (blok[i].handle == NULL) {	// slobodan?
					// uveca broj alociranih blokova
					++num;
					// uveca koliko je ukupno alocirano
					sum += size;
					blok[i].handle = handle;
					blok[i].size = size;
					break;
				}
			}

			Unlock();	// izadje iz krit. odsjecka
		}

		bool RemoveData(HGLOBAL handle) {
			Lock();

			bool nasao = false;

			// trazi blok kojeg se delita
			for (int i = 0; i < NUM_BLOKOVA; ++i) {
				if (blok[i].handle == handle) {	// naso?
					sum -= blok[i].size;
					num--;
					blok[i].handle = NULL;
					nasao = true;
					break;
				}
			}

			Unlock();
			return nasao;
		}

	private:
		unsigned int sum;
		unsigned int num;

		struct {
			HANDLE handle;
			unsigned int size;
		} blok[NUM_BLOKOVA];
} nmd;

static void FillDefaults(Report *r) {
	unsigned int num, sum;
	sum = nmd.GetSum();
	num = nmd.GetNum();
	r->AddInt("sum", sum);		// sum(velicina) svih alociranih
	r->AddInt("num", num);		// N alociranih
	r->AddInt("avg", sum/num);	// prosjek
}

HGLOBAL __stdcall nAPI_GlobalAlloc(UINT uFlags, SIZE_T dwBytes) {
	Report r("GlobalAlloc");	// deklaracija izvjesca
	HGLOBAL result;

	// poziv na originalni api
	result = GlobalAlloc(uFlags, dwBytes);
	dprintf("GlobalAlloc(0x%08X, %u) = %p\n", uFlags, (unsigned int)dwBytes, result);

	// obnovimo globalne podatke
	if (result != NULL) {	// ako je uspjesno alocirano!
		nmd.AddData(result, (unsigned int)dwBytes);
	} else {
		r.AddString("greska", ErrorDescription());
	}

	// generiranje izvjesca specificnog za GlobalAlloc
	r.AddHex("flags", uFlags);
	r.AddInt("size", (unsigned int)dwBytes);
	r.AddPtr("handle", result);

	// popunjavanje izvjesca sa defaultnim vrijednostima
	FillDefaults(&r);

	// dodavanje izvjesca u log
	AddToLog(r);

	// vracanje rezultata originalnog apija
	return result;
}

HGLOBAL __stdcall nAPI_GlobalFree(HGLOBAL hMem) {
	Report r("GlobalFree");
	HGLOBAL result;
	bool nadjen = false;

	result = GlobalFree(hMem);
	dprintf("GlobalFree(%p) = %p\n", hMem, result);

	if (result == NULL) {	// GlobalFree je uspio
		nadjen = nmd.RemoveData(hMem);
	} else {
		r.AddString("greska", ErrorDescription());
	}

	r.AddPtr("handle", hMem);
	r.AddPtr("result", result);
	r.AddInt("found", nadjen);

	FillDefaults(&r);
	AddToLog(r);

	return result;
}
