#include<winsock2.h>
#include<windows.h>
#include<ws2tcpip.h>

#include "SocketAPIOverride.h"
#include "debugio.h"
#include "Locker.h"
#include "Log.h"

#define NUM_BLOKOVA 100

// klasa sa globalnim podacima za socket apije
static class nAPISocketData : private Locker {
public:
	nAPISocketData() {
		numSocket = 0;
		sentBytes = 0;
		recvBytes = 0;
		sentnum = 0;
		recvnum = 0;
		ZeroMemory(blok, sizeof(blok));
		
	}

	unsigned int GetNum() {
		return numSocket;
	}

	unsigned int GetSent() {
		return sentBytes;
	}

	unsigned int GetRecv() {
		return recvBytes;
	}

	unsigned int GetRecvNum() {
		return recvnum;
	}

	unsigned int GetSentNum() {
		return sentnum;
	}

	LARGE_INTEGER GetStart(SOCKET socket) {
		unsigned int i;
		LARGE_INTEGER nosocket;

		for (i = 0; i < NUM_BLOKOVA; i++)
			if (blok[i].socket == socket)
				return blok[i].tick1;

		ZeroMemory(&nosocket, sizeof(LARGE_INTEGER));
		return nosocket;
	}

	LARGE_INTEGER GetStop(SOCKET socket) {
		unsigned int i;
		LARGE_INTEGER nosocket;

		for (i = 0; i < NUM_BLOKOVA; i++)
			if (blok[i].socket == socket)
				return blok[i].tick2;

		ZeroMemory(&nosocket, sizeof(LARGE_INTEGER));
		return nosocket;
	}

	LARGE_INTEGER GetFrequency(SOCKET socket) {
		unsigned int i;
		LARGE_INTEGER nosocket;

		for (i = 0; i < NUM_BLOKOVA; i++)
			if (blok[i].socket == socket)
				return blok[i].tickspersec;

		ZeroMemory(&nosocket, sizeof(LARGE_INTEGER));
		return nosocket;
	}

	bool GetVr(SOCKET socket) {
		unsigned int i;

		for (i = 0; i < NUM_BLOKOVA; i++)
			if (blok[i].socket == socket)
				return blok[i].vr;

		return false;
	}

	void StartCounting(SOCKET socket) {
		unsigned int i;

		Lock();
		// trazi socket za pocetak stopanja
		for (i = 0; i < NUM_BLOKOVA; i++) {
			if (blok[i].socket == socket) {
				blok[i].vr = true;
				QueryPerformanceFrequency(&blok[i].tickspersec);
				QueryPerformanceCounter(&blok[i].tick1);
				break;
			}
		}
		Unlock();
	}

	void StopCounting(SOCKET socket) {
		unsigned int i;

		Lock();
		// trazi socket za zavrsetak stopanja
		for (i = 0; i < NUM_BLOKOVA; i++) {
			if (blok[i].socket == socket) {
				QueryPerformanceCounter(&blok[i].tick2);
				break;
			}
		}
		Unlock();
	}

	void NewSocket(SOCKET socket) {
		unsigned int i;
		
		Lock();
		numSocket++;
		for (i = 0; i < NUM_BLOKOVA; i++) {
			if (blok[i].socket == NULL) {
				printf("\nDodajem socket %u na poziciji %d\n\n", socket, i);
				blok[i].socket = socket;
				break;
			}
		}
		Unlock();
	}

	void DeleteSocket(SOCKET socket) {
		unsigned int i;

		for (i = 0; i < NUM_BLOKOVA; i++) {
			if (blok[i].socket == socket) {
				printf("\nBrisem socket %u na poziciji %d\n\n", socket, i);
				ZeroMemory(&blok[i], sizeof(blok[i]));
				break;
			}
		}
	}

	void SentData(int size) {
		Lock();
		sentnum++;
		sentBytes += size;
		Unlock();
	}

	void RecvData(int size) {
		Lock();
		recvnum++;
		recvBytes += size;
		Unlock();
	}

private:
	unsigned int numSocket;
	unsigned int sentBytes, sentnum;
	unsigned int recvBytes, recvnum;
	struct {
		SOCKET socket;
		bool vr;
		LARGE_INTEGER tickspersec, tick1, tick2;
	} blok[NUM_BLOKOVA];
} nmd;

/*
static void FillDefaults(Report *r) {
}
*/

void GetError(char *buffer) {
	int error;

	error = WSAGetLastError();

	switch(error) {
		case WSAEPROTONOSUPPORT:
			sprintf(buffer, "Protokol nije podrzan.");
			break;
		case WSAENOTSOCK:
			sprintf(buffer, "Operacija sa socketima pokusana na nesocketu.");
			break;
		case WSAEINVAL:
			sprintf(buffer, "Krivi argument funkcije.");
			break;
		case WSAECONNREFUSED:
			sprintf(buffer, "Konekcija odbijena.");
			break;
		case WSAEADDRINUSE:
			sprintf(buffer, "Adresa se vec koristi.");
			break;
		case WSAECONNRESET:
			sprintf(buffer, "Konekcija resetirana.");
			break;
		case 0:
			sprintf(buffer, "Nepoznata greska.");
			break;
		default:
			sprintf(buffer, "Greska: %d.", error);
	}

}


// kreiranje socketa
SOCKET __stdcall nAPI_socket(int af, int type, int protocol) {
	Report report("socket");
	SOCKET result;
	char buffer[100];

	// socket vraca opisnik novog socketa, a inace INVALID_SOCKET
	result = socket(af, type, protocol); 
	dprintf("socket(%08X, %08X, %08X) = %u\n", af, type, protocol, result);

	// ako socket ne vrati INVALID_SOCKET, socket() je uspjela
	if (result != INVALID_SOCKET)  {
		report.AddInt("success", 1);
		nmd.NewSocket(result);
		report.AddString("greska", "0");
	}
	else {
		report.AddInt("success", 0);
		GetError(buffer);
		report.AddString("greska", buffer);
	}

	// ako se koristi UDP onda se trajanje veze racuna od kreiranja socketa
	// if (type == SOCK_DGRAM)
	nmd.StartCounting(result);

	report.AddUInt("socket", (UINT) result);
	report.AddHex("family", af);
	report.AddInt("type", type);
	report.AddInt("protocol", protocol);
	report.AddUInt("brojsocketa", nmd.GetNum());

	// FillDefaults(&report);
	AddToLog(report);

	return result;
};


// slanje za TCP protokol
int __stdcall nAPI_send(SOCKET socket, const char *sentdata, int size, int flags) {
	Report report("send");
	int result, sentnum, sentBytes;
	char buffer[100];
	SYSTEMTIME systime;
	double brzina;

	LARGE_INTEGER ticksPerSecond;
	LARGE_INTEGER tick1, tick2;
	LARGE_INTEGER time;

	// funkcija stavlja u ticksPerSecond frekvenciju
	QueryPerformanceFrequency(&ticksPerSecond);
	// u tick1 funkcija stavlja broj ciklusa od pokretanja sustava
	QueryPerformanceCounter(&tick1);

	// send vraca broj poslanih byteova, a inace SOCKET_ERROR
	result = send(socket, sentdata, size, flags);

	// broj ciklusa od pokretanja sustava pa odmah do poslije send()
	QueryPerformanceCounter(&tick2); 

	// nakon sto je send poslao, gleda se trenutacno vrijeme i datum
	GetLocalTime(&systime);

	sprintf(buffer, "%02hu.%02hu.%hu. %02hu:%02hu:%02hu", systime.wDay, systime.wMonth, 
		systime.wYear, systime.wHour, systime.wMinute, systime.wSecond);

	report.AddString("vrslanja", buffer);

	dprintf("send(%u, %p, %d, %08X) = %d\n", socket, sentdata, size, flags, result);

	// u tick2 je broj otkucaja od pokretanja sustava pa sve do nakon izvrsavanja fje
	// u tick1 je broj od pokretanja sustava do prije izvrsavanja fje
	// razlika izmedju tick2 i tick1 se podijeli s frekvencijom
	// pomnozi se sa 10^6 da bismo dobili vrijeme u mikrosekundama
	time.QuadPart = (tick2.QuadPart-tick1.QuadPart)*1000000/ticksPerSecond.QuadPart;

	// send() je uspjela, racuna se brzina
	if (result != SOCKET_ERROR) {
		nmd.SentData(result);
		report.AddInt("success", 1);

		// broj byteova mnozimo s 10^6 i rezultat je u bps
		brzina = (double)result*1000000/(double)time.QuadPart; 
		report.AddDouble("brzina", brzina);
		report.AddInt("poslano", result);
		report.AddString("greska", "0");
	}
	// ako send() nije uspjela, brzina slanja postavlja se na 0
	else {
		report.AddInt("success", 0);
		report.AddDouble("brzina", 0);
		report.AddInt("poslano", 0);
		GetError(buffer);
		report.AddString("greska", buffer);
	}

	sentnum = nmd.GetSentNum();
	sentBytes = nmd.GetSent();
	report.AddUInt("brojslanja", sentnum);
	report.AddUInt("ukposlano", sentBytes);

	if (sentnum == 0)
		report.AddInt("avgposl", 0);
	else
		report.AddInt("avgposl", sentBytes/sentnum);

	report.AddUInt("vrijeme", (UINT) time.LowPart);
	report.AddUInt("socket", (UINT) socket);
	report.AddHex("flags", flags);
	report.AddPtr("sendptr", (void*)sentdata);

	// FillDefaults(&report);
	AddToLog(report);

	return result;

};

// primanje podataka za TCP protokol
int __stdcall nAPI_recv(SOCKET socket, char *buf, int size, int flags) {
	Report report("recv");
	int result, recvnum, recvBytes;
	char buffer[100];
	SYSTEMTIME systime;
	double brzina;

	LARGE_INTEGER ticksPerSecond;
	LARGE_INTEGER tick1, tick2;
	LARGE_INTEGER time;

	QueryPerformanceFrequency(&ticksPerSecond);
	QueryPerformanceCounter(&tick1);

	// vraca broj primljenih byteova, inace SOCKET_ERROR
	result = recv(socket, buf, size, flags);

	QueryPerformanceCounter(&tick2);

	// nakon sto je send poslao, gleda se trenutacno vrijeme i datum
	GetLocalTime(&systime);

	sprintf(buffer, "%02hu.%02hu.%hu. %02hu:%02hu:%02hu", systime.wDay, systime.wMonth, 
		systime.wYear, systime.wHour, systime.wMinute, systime.wSecond);

	report.AddString("vrprimanja", buffer);

	time.QuadPart = (tick2.QuadPart-tick1.QuadPart)*1000000/ticksPerSecond.QuadPart;

	dprintf("recv(%u, %p, %u, %08X) = %d\n", socket, buf, size, flags, result);

	// recv() je uspjela, racuna se brzina
	if (result != SOCKET_ERROR) {
		nmd.RecvData(result);
		report.AddInt("success", 1);
		// broj byteova mnozimo s 10^6 i rezultat je u bps
		brzina = (double)result*1000000/(double)time.QuadPart;
		report.AddDouble("brzina", brzina);
		report.AddInt("primljeno", result);
		report.AddString("greska", "0");
	}
	// recv() nije uspjela, brzina je 0
	else {
		report.AddInt("success", 0);
		report.AddDouble("brzina", 0);
		report.AddInt("primljeno", 0);
		GetError(buffer);
		report.AddString("greska", buffer);
	}

	recvBytes = nmd.GetRecv();
	recvnum = nmd.GetRecvNum();
	report.AddUInt("ukprimljeno", recvBytes);
	report.AddUInt("brojprimanja", recvnum);

	if (recvnum == 0)
		report.AddInt("avgprim", 0);
	else
		report.AddInt("avgprim", recvBytes/recvnum);

	report.AddUInt("vrijeme", (UINT) time.LowPart);
	report.AddUInt("socket", (UINT) socket);
	report.AddHex("flags", flags);
	report.AddPtr("recvptr", buf);

	// FillDefaults(&report);
	AddToLog(report);

	return result;
};

SOCKET __stdcall nAPI_accept(SOCKET socket, struct sockaddr* addr, int *addrlen) {
	Report report("accept");
	SOCKET result;
	int port;
	char buffer[100];

	// accept vraca vrijednost koja je opisnik novog socketa, inace vraca INVALID_SOCKET
	result = accept(socket, addr, addrlen);
	dprintf("accept(%u, %p, %p) = %d\n", socket, addr, addrlen, result);

	if (result != INVALID_SOCKET) {
		report.AddInt("success", 1);
		nmd.NewSocket(result);
		report.AddString("greska", "0");
	}
	else {
		report.AddInt("success", 0);
		GetError(buffer);
		report.AddString("greska", buffer);
	}

	nmd.StartCounting(result);

	report.AddUInt("newsocket", (UINT) result);
	report.AddUInt("listensocket", (UINT) socket);

	if (addr != NULL) {
		sprintf(buffer, "%d.%d.%d.%d", 
			addr->sa_data[2] & 0xff, addr->sa_data[3] & 0xff, addr->sa_data[4] & 0xff, 
			addr->sa_data[5] & 0xff);

		report.AddString("adresa", buffer);

		port = ((addr->sa_data[0] & 0xff) << 8) | addr->sa_data[1] & 0xff;

		report.AddInt("port", port); 
	}
	else {
		report.AddString("adresa", NULL);
		report.AddInt("port", 0);
	}
	report.AddPtr("addrlen", addrlen);
	report.AddUInt("brojsocketa", nmd.GetNum());

	// FillDefaults(&report);
	AddToLog(report);

	return result;

};

int _stdcall nAPI_bind(SOCKET socket, const struct sockaddr *name, int namelen) {
	Report report("bind");
	int result;
	char buffer[100];
	int port;

	// vraca 0 ako uspije, inace SOCKET_ERROR
	result = bind(socket, name, namelen);
	dprintf("bind(%u, %p, %d) = %d\n", socket, name, namelen, result); 

	if (result == 0) {
		report.AddInt("success", 1);
		report.AddString("greska", "0");
	}
	else {
		report.AddInt("success", 0);
		GetError(buffer);
		report.AddString("greska", buffer);
	}

	report.AddUInt("socket", (UINT) socket);

	sprintf(buffer, "%d.%d.%d.%d", name->sa_data[2] & 0xff, name->sa_data[3] & 0xff, 
		name->sa_data[4] & 0xff , name->sa_data[5] & 0xff);

	report.AddString("adresa", buffer);

	port = ((name->sa_data[0] & 0xff) << 8) | name->sa_data[1] & 0xff;

	report.AddInt("port", port);
	report.AddInt("namelen", namelen);

	// FillDefaults(&report);
	AddToLog(report);

	return result;

};

int _stdcall nAPI_listen(SOCKET socket, int backlog) {
	Report report("listen");
	int result;
	char buffer[100];

	// ako uspije vraca 0, inace SOCKET_ERROR
	result = listen(socket, backlog); 
	dprintf("listen(%u, %d) = %d\n", socket, backlog, result);

	if (result == 0) {
		report.AddInt("success", 1);
		report.AddString("greska", "0");
	}
	else {
		report.AddInt("success", 0);
		GetError(buffer);
		report.AddString("greska", buffer);
	}

	report.AddUInt("socket", (UINT) socket);
	report.AddInt("maxconnect", backlog);

	// FillDefaults(&report);
	AddToLog(report);

	return result;

};

int __stdcall nAPI_connect(SOCKET socket, const struct sockaddr *name, int namelen) {
	Report report("connect");
	int result;
	char buffer[100];
	int port;

	// vraca 0 ako nema errora, inace SOCKET_ERROR
	result = connect(socket, name, namelen); 
	dprintf("connect(%u, %p, %d) = %d\n", socket, name, namelen, result); 

	nmd.StartCounting(socket);

	if (result == 0) {
		report.AddInt("success", 1);
		report.AddString("greska", "0");
	}
	else {
		report.AddInt("success", 0);
		GetError(buffer);
		report.AddString("greska", buffer);
	}

	report.AddUInt("socket", (UINT) socket);

	sprintf(buffer, "%d.%d.%d.%d", name->sa_data[2] & 0xff, name->sa_data[3] & 0xff, 
		name->sa_data[4] & 0xff , name->sa_data[5] & 0xff);

	report.AddString("adresa", buffer);

	port = ((name->sa_data[0] & 0xff) << 8) | name->sa_data[1] & 0xff;

	report.AddInt("port", port);
	report.AddInt("namelen", namelen);

	// FillDefaults(&report);
	AddToLog(report);

	return result;

};

int __stdcall nAPI_closesocket(SOCKET socket) {
	Report report("closesocket");
	int result, sentBytes, recvBytes;
	LARGE_INTEGER start, stop, frequency;
	bool vr;
	double vrijeme;
	char buffer[100];

	// vraca 0 ako nema errora, inace SOCKET_ERROR
	result = closesocket(socket);
	dprintf("closesocket(%u) = %d\n", socket, result);

	nmd.StopCounting(socket);

	sentBytes = nmd.GetSent();
	recvBytes = nmd.GetRecv();
	vr = nmd.GetVr(socket);
	start = nmd.GetStart(socket);
	stop = nmd.GetStop(socket);
	frequency = nmd.GetFrequency(socket);

	if (result == 0) {
		report.AddInt("success", 1);
		report.AddString("greska", "0");
		nmd.DeleteSocket(socket);
	}
	else {
		report.AddInt("success", 0);
		GetError(buffer);
		report.AddString("greska", buffer);
	}

	// vr oznacava postoji li pocetna vrijednost vremena
	// tcp: vrijeme od connect() do closesocket()
	// udp: vrijeme od socket() do closesocket()
	if (vr == true) {
		// vrijeme u mikrosekundama
		vrijeme = (double)(stop.QuadPart-start.QuadPart)*1000000/frequency.QuadPart;
		report.AddDouble("ukvrijeme", vrijeme);
		// brzina slanja u bps
		report.AddDouble("avgbrsl", (double) sentBytes*1000000/ vrijeme);
		// brzina primanja u bps
		report.AddDouble("avgbrpr", (double) recvBytes*1000000/ vrijeme);
	}
	else {
		report.AddDouble("avgbrsl", 0);
		report.AddDouble("avgbrpr", 0);
		report.AddDouble("ukvrijeme", 0);
	}

	report.AddUInt("socket", (UINT) socket);

	// FillDefaults(&report);
	AddToLog(report);

	return result;

};

int __stdcall nAPI_sendto(SOCKET socket, const char *buf, int len, int flags, 
						  const sockaddr *sendaddr, int tolen) {
	Report report("sendto");
	int result, sentnum, sentBytes;
	char buffer[100];
	SYSTEMTIME systime;
	int port;
	double brzina;

	LARGE_INTEGER ticksPerSecond;
	LARGE_INTEGER tick1, tick2;
	LARGE_INTEGER time;  

	QueryPerformanceFrequency(&ticksPerSecond);
	QueryPerformanceCounter(&tick1);

	// sendto() vraca broj poslanih byteova, a ako ne uspije SOCKET_ERROR
	result = sendto(socket, buf, len, flags, sendaddr, tolen);

	QueryPerformanceCounter(&tick2);

	// nakon sto je send poslao, gleda se trenutacno vrijeme i datum
	GetLocalTime(&systime);

	sprintf(buffer, "%02hu.%02hu.%hu. %02hu:%02hu:%02hu", systime.wDay, systime.wMonth, 
		systime.wYear, systime.wHour, systime.wMinute, systime.wSecond);

	report.AddString("vrslanja", buffer);

	time.QuadPart = (tick2.QuadPart-tick1.QuadPart)*1000000/ticksPerSecond.QuadPart;


	dprintf("sendto(%u, %p, %d, %08X, %p, %d) = %d\n", 
		socket, buf, len, flags, sendaddr, tolen, result);

	// sendto() je uspjela, racuna se brzina
	if (result != SOCKET_ERROR) { 
		nmd.SentData(result);
		report.AddInt("success", 1);
		// broj byteova mnozimo s 10^6 i rezultat je u bps
		brzina = (double)result*1000000/(double)time.QuadPart;
		report.AddDouble("brzina", brzina);
		report.AddInt("poslano", result);
		report.AddString("greska", "0");
	}
	// sendto() nije uspjela, brzina je 0
	else {
		report.AddInt("success", 0);
		report.AddDouble("brzina", 0);
		report.AddInt("poslano", 0);
		GetError(buffer);
		report.AddString("greska", buffer);
	}

	sentnum = nmd.GetSentNum();
	sentBytes = nmd.GetSent();
	report.AddUInt("brojslanja", sentnum);
	report.AddUInt("ukposlano", sentBytes);

	if (sentnum == 0)
		report.AddInt("avgposl", 0);
	else
		report.AddInt("avgposl", sentBytes/sentnum);

	report.AddUInt("vrijeme", (UINT) time.LowPart);
	report.AddUInt("socket", (UINT) socket);
	report.AddPtr("sendptr", (char*)buf);
	report.AddHex("flags", flags);

	sprintf(buffer, "%d.%d.%d.%d", 
		sendaddr->sa_data[2] & 0xff, sendaddr->sa_data[3] & 0xff, 
		sendaddr->sa_data[4] & 0xff , sendaddr->sa_data[5] & 0xff);

	report.AddString("adresa", buffer);

	port = ((sendaddr->sa_data[0] & 0xff) << 8) | sendaddr->sa_data[1] & 0xff;

	report.AddInt("port", port);
	report.AddInt("addrlen", tolen);

	// FillDefaults(&report);
	AddToLog(report);

	return result;
}


int __stdcall nAPI_recvfrom(SOCKET socket, char *buf, int len, int flags, 
							struct sockaddr *recvaddr, int *recvaddrlen) {
	Report report("recvfrom");
	int result, recvnum, recvBytes;
	char buffer[100];
	SYSTEMTIME systime;
	int port;
	double brzina;

	LARGE_INTEGER ticksPerSecond;
	LARGE_INTEGER tick1, tick2;
	LARGE_INTEGER time;  

	QueryPerformanceFrequency(&ticksPerSecond);
	QueryPerformanceCounter(&tick1);

	// recvfrom() vraca broj primljenih byteova, a inace SOCKET_ERROR 
	result = recvfrom(socket, buf, len, flags, recvaddr, recvaddrlen);

	QueryPerformanceCounter(&tick2);

	// nakon sto je recv primio, gleda se trenutacno vrijeme i datum
	GetLocalTime(&systime);

	sprintf(buffer, "%02hu.%02hu.%hu. %02hu:%02hu:%02hu", systime.wDay, systime.wMonth, 
		systime.wYear, systime.wHour, systime.wMinute, systime.wSecond);

	report.AddString("vrprimanja", buffer);

	time.QuadPart = (tick2.QuadPart-tick1.QuadPart)*1000000/ticksPerSecond.QuadPart;

	dprintf("recvfrom(%u, %p, %d, %08X, %p, %p) = %d\n", 
		socket, buf, len, flags, recvaddr, recvaddrlen, result);

	// recvfrom() je uspjela, racuna se brzina
	if (result != SOCKET_ERROR) {
		nmd.RecvData(result);
		report.AddInt("success", 1);
		// broj byteova mnozimo s 10^6 i rezultat je u bps
		brzina = (double)result*1000000/(double)time.QuadPart;
		report.AddDouble("brzina", brzina);
		report.AddInt("primljeno", result);
		report.AddString("greska", "0");
	}
	// recvfrom() nije uspjela, brzina je 0
	else {
		report.AddInt("success", 0);
		report.AddDouble("brzina", 0);
		report.AddInt("primljeno", 0);
		GetError(buffer);
		report.AddString("greska", buffer);
	}

	recvBytes = nmd.GetRecv();
	recvnum = nmd.GetRecvNum();
	report.AddUInt("ukprimljeno", recvBytes);
	report.AddUInt("brojprimanja", recvnum);

	if (recvnum == 0)
		report.AddInt("avgprim", 0);
	else
		report.AddInt("avgprim", recvBytes/recvnum);

	report.AddUInt("vrijeme", (UINT) time.LowPart); 
	report.AddUInt("socket", (UINT) socket);

	sprintf(buffer, "%d.%d.%d.%d", 
		recvaddr->sa_data[2] & 0xff, recvaddr->sa_data[3] & 0xff, 
		recvaddr->sa_data[4] & 0xff, recvaddr->sa_data[5] & 0xff);

	report.AddString("adresa", buffer);

	port = ((recvaddr->sa_data[0] & 0xff) << 8) | recvaddr->sa_data[1] & 0xff;

	report.AddInt("port", port);
	report.AddHex("flags", flags);
	report.AddPtr("recvptr", (char*)recvaddr);
	report.AddPtr("addrlen", recvaddrlen);

	// FillDefaults(&report);
	AddToLog(report);

	return result;

}
