﻿using System;
using System.IO;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace nAPI
{
    public partial class Form1 : Form
    {
        ToolTip tt = new ToolTip();

        bool nadzorUTijeku = false;
        bool prviPut = true;

        bool tab2On = false;
        bool tab3On = false;
        bool tab4On = false;

        string popisPotrebnihKontrola = "";
        string linija = "______________________________________________________________\r\n";

        delegate void SetTextCallback(string text, byte indeksTaba);

        StreamReader jezgraOutput;


        public Form1()
        {            
            InitializeComponent();

            foreach (TreeNode ChildNode in treeView1.Nodes)
            {
                ChildNode.Expand();
            }
        }
        
        /* Metoda treeView1_AfterCheck poziva se prilikom svakog "čekiranja" unutar treeView1.
         * Čvorovima djeci čekiranog čvora roditelja postavlja check u ovisnosti o stanju roditelja.
         */
        private void treeView1_AfterCheck(object sender, TreeViewEventArgs e)
        {
            e.Node.ExpandAll();

            TreeNodeCollection col = e.Node.Nodes; // U col spremi čvorove djecu čvora koji je upravo čekiran
            foreach (TreeNode t in col)
            {
                t.Checked = e.Node.Checked;
            }
        }



        private void pokreniNovoNadziranje()
        {
            sakrijKontrole();

            Konfiguracija konf = new Konfiguracija(treeView1);
            int brojOpcija = konf.zapisiKofiguraciju();

            DialogResult dr = new DialogResult();
            if (brojOpcija == 0)
            {
                dr = MessageBox.Show(
                    "Niti jedna opcija nadziranja nije označena. Pokrenuta aplikacija neće" +
                    " biti nadzirana.\r\n" +
                    "Želite li nastaviti?",
                    "Upozorenje", MessageBoxButtons.YesNo,
                    MessageBoxIcon.Warning,
                    MessageBoxDefaultButton.Button2);
            }

            if (!dr.Equals(DialogResult.No))
            {
                openFileDialog1.InitialDirectory = Environment.GetFolderPath
                    (Environment.SpecialFolder.Programs);

                if (openFileDialog1.ShowDialog() != DialogResult.Cancel)
                {
                    if (dr.Equals(DialogResult.Yes))
                        richTextBox1.Text = "Aplikacija je pokrenuta u nenadziranom načinu.\r\n\r\n";
                    else
                        richTextBox1.Text = "Jezgra je uspješno konfigurirana. Aplikacija je" +
                            " pokrenuta u nadziranom načinu.\r\n\r\n";

                    nadzorUTijeku = true;


                    System.Diagnostics.Process proc = new System.Diagnostics.Process();

                    proc.StartInfo.FileName = "Starter.exe";
                    proc.StartInfo.Arguments = "\"" + openFileDialog1.FileName + "\"";

                    proc.StartInfo.UseShellExecute = false;
                    proc.StartInfo.CreateNoWindow = true;
                    proc.StartInfo.RedirectStandardOutput = true;

                    proc.EnableRaisingEvents = true;
                    proc.Exited += new EventHandler(proc_Exited);

                    proc.Start();

                    jezgraOutput = proc.StandardOutput;


                    popisPotrebnihKontrola = konf.pronadjiPotrebneKontrole();
                    prikaziPotrebneKontrole(popisPotrebnihKontrola);
                }
            }
        }

    
        private void proc_Exited(object sender, EventArgs e)
        {
            nadzorUTijeku = false;
            
            int pom = openFileDialog1.FileName.LastIndexOf("\\") + 1;
            Log mojLog = new Log(openFileDialog1.FileName.Substring(pom));
            mojLog.ucitajLog();

            string ispis;
            string[] statDijelovi;

            SetTextCallback d = new SetTextCallback(ispisiTekst);
            for (byte i = 1; i <= 4; i++)
            {
                ispis = mojLog.ispisiUcitaniLog(i);

                // Komponenta richTextBox na koju trebamo ispisivati izvodi se u drugoj dretvi 
                // pa radi sigurnosti koristimo Invoke.
                this.Invoke(d, new object[] { ispis + "\r\n", i });
            }

            d = new SetTextCallback(ispisiStatistiku);
            for (byte i = 2; i <= 4; i++)
            {
                ispis = mojLog.ispisiStat(i);
                //MessageBox.Show(i.ToString() + ":\r\n[" + ispis + "]");
                statDijelovi = ispis.Split(new char[] { '\n' }, StringSplitOptions.RemoveEmptyEntries);
                foreach (string statDio in statDijelovi)
                {
                    this.Invoke(d, new object[] { statDio.Trim() + "\r\n", i });
                }
            }
        }

        private void ispisiTekst(string tekst, byte indTaba)
        {
            switch (indTaba)
            {
                case 1:
                    richTextBox1.Text += linija;
                    richTextBox1.Text += "----------Aktivnosti u jezgri----------\r\n\r\n";
                    richTextBox1.Text += jezgraOutput.ReadToEnd() + "\r\n\r\n";

                    richTextBox1.Text += linija;
                    richTextBox1.Text += "Pri nadziranju API-ja zabilježene su sljedeće funkcije:\r\n";
                    richTextBox1.Text += tekst;
                    break;
                case 2:
                    richTextBox2.Text += "Pri nadziranju API-ja zabilježene su sljedeće datotečne funkcije:\r\n\r\n";
                    richTextBox2.Text += tekst;
                    break;
                case 3:
                    richTextBox3.Text += "Pri nadziranju API-ja zabilježene su sljedeće mrežne funkcije:\r\n\r\n";
                    richTextBox3.Text += tekst;
                    break;
                case 4:
                    richTextBox4.Text += "Pri nadziranju API-ja zabilježene su sljedeće memorijske funkcije:\r\n\r\n";
                    richTextBox4.Text += tekst;
                    break;
            }
        }
        
        private void ispisiStatistiku(string stat, byte indTaba)
        {
            int pom = stat.IndexOf(' ');
            string tipStat = stat.Substring(0, pom);
            stat = stat.Substring(pom + 1);

            pom = stat.IndexOf(':');
            string podtipStat = stat.Substring(0, pom);
            stat = stat.Substring(pom + 2);
            stat = stat.Trim();

            //MessageBox.Show("[" + tipStat + "] [" + podtipStat + "] [" + stat + "]");

            switch (indTaba)
            {
                case 2:
                    {
                        if (tipStat == "ReadFile")
                        {
                            foreach (Control con in groupBox2.Controls)
                            {
                                if (con.Name == podtipStat)
                                    con.Text = stat;
                            }
                        }
                        else if (tipStat == "WriteFile")
                        {
                            foreach (Control con in groupBox3.Controls)
                            {
                                if (con.Name == podtipStat)
                                    con.Text = stat;
                            }
                        }
                    }
                    break;
                case 3:
                    {
                        if (tipStat == "recv" || tipStat == "recvfrom")
                        {
                            foreach (Control con in groupBox1.Controls)
                            {
                                if (con.Name == podtipStat)
                                    con.Text = stat;
                            }
                        }
                        else if (tipStat == "send" || tipStat == "sendto")
                        {
                            foreach (Control con in groupBox4.Controls)
                            {
                                if (con.Name == podtipStat)
                                    con.Text = stat;
                            }
                        }
                    }
                    break;
                case 4:
                    if (tipStat == "GlobalAlloc")
                    {
                        foreach (Control con in groupBox9.Controls)
                        {
                            if (con.Name == podtipStat + "A")
                                con.Text = stat;
                        }
                    }
                    else if (tipStat == "GlobalFree")
                    {
                        foreach (Control con in groupBox10.Controls)
                        {
                            if (con.Name == podtipStat + "F")
                                con.Text = stat;
                        }
                    }
                    break;
            }
        }

        
        private void prikaziPotrebneKontrole(string potrebneKontrole)
        {
            string buffer = "";

            for (int i = 0; i < potrebneKontrole.Length; i++)
            {
                while (potrebneKontrole[i] != ' ' && potrebneKontrole[i] != '\r' && potrebneKontrole[i] != '\n')
                {
                    buffer += potrebneKontrole[i];
                    i++;
                    if (i >= potrebneKontrole.Length) break;
                }

                switch (buffer)
                {
                    case "Datoteke":
                        this.tabControl1.Controls.Add(this.tabPage2);
                        tab2On = true;                           
                        break;
                    case "Mreže":
                        this.tabControl1.Controls.Add(this.tabPage3);
                        tab3On = true;
                        break;
                    case "Memorija":
                        this.tabControl1.Controls.Add(this.tabPage4);
                        tab4On = true;
                        break;
                    case "ReadFile":
                        groupBox2.Visible = true;
                        break;
                    case "WriteFile":
                        groupBox3.Visible = true;
                        break;
                    case "recv":
                        groupBox1.Visible = true;
                        break;
                    case "recvfrom":
                        groupBox1.Visible = true;
                        break;
                    case "send":
                        groupBox4.Visible = true;
                        break;
                    case "sendto":
                        groupBox4.Visible = true;
                        break;
                    case "GlobalAlloc":
                        groupBox9.Visible = true;
                        break;
                    case "GlobalFree":
                        groupBox10.Visible = true;
                        break;
                }

                buffer = "";
                    
            }
        }

        private void sakrijKontrole()
        {
            if (tab2On)
            {
                tabControl1.Controls.Remove(tabPage2);
                tab2On = false;
            }
            if (tab3On)
            {
                tabControl1.Controls.Remove(tabPage3);
                tab3On = false;
            }
            if (tab4On)
            {
                tabControl1.Controls.Remove(tabPage4);
                tab4On = false;
            }

            groupBox2.Visible = false;
            groupBox3.Visible = false;
            groupBox1.Visible = false;
            groupBox1.Visible = false;
            groupBox4.Visible = false;
            groupBox4.Visible = false;
            groupBox9.Visible = false;
            groupBox10.Visible = false;
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            //if (indikator)
            //    tr.Close();            
        }


        private void treeView1_BeforeCheck(object sender, TreeViewCancelEventArgs e)
        {
            if (nadzorUTijeku && prviPut)
            {
                MessageBox.Show("Proces nadziranja već je započeo i označavanje opcija nadziranja" + 
                    " više nema utjecaja na nadziranje pokrenute aplikacije.\r\nUkoliko želite " + 
                    "započeti novo nadziranje ugasite nadziranu aplikaciju i pritisnite gumb " + 
                    "za novo nadziranje.",
                    "Pažnja!",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Hand);

                prviPut = false;
            }
        }


        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            pokreniNovoNadziranje();
        }


        private void označisveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TreeNodeCollection col = treeView1.Nodes;

            foreach (TreeNode tn in col)
            {
                oznaciSve(tn);
            }
        }

        private void oznaciSve(TreeNode node)
        {
            node.Checked = true;

            if (node.Nodes.Count > 0)
            {
                foreach (TreeNode childNode in node.Nodes)
                {
                    childNode.Checked = true;
                    oznaciSve(childNode);
                }
            }
        }

        private void označiDatotekeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TreeNodeCollection col = treeView1.Nodes;

            foreach (TreeNode tn in col)
            {
                if (tn.Text == "Datoteke")
                oznaciSve(tn);
            }
        }

        private void označiMrežeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TreeNodeCollection col = treeView1.Nodes;

            foreach (TreeNode tn in col)
            {
                if (tn.Text == "Mreže")
                    oznaciSve(tn);
            }
        }

        private void označiMemorijuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TreeNodeCollection col = treeView1.Nodes;

            foreach (TreeNode tn in col)
            {
                if (tn.Text == "Memorija")
                    oznaciSve(tn);
            }
        }


        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            tt.SetToolTip(treeView1, e.Node.ToolTipText);
            tt.Active = true;

            tt.Show(e.Node.ToolTipText, treeView1, 
                e.Node.Bounds.X + 20, e.Node.Bounds.Y + 20, 3000);         
        }


        private void contentsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Upute uputeZaRuk = new Upute(treeView1);
            uputeZaRuk.Show();
        }


        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show(
                    "Sustav nAPI je razvijen u sklopu predmeta Projekt iz programske potpore" +
                    " na Fakultetu elektrotehnike i računarstva.\r\n" +
                    "Mentor: dr.sc. Leonardo Jelenković\r\n" +
                    "Autori: Davor Cihlar, Berislav Hunjadi, Neven Ćubić, " +  
                    "Ognjen Lajšić, Hrvoje Knežević, Marko Plačko",
                    "O sustavu nAPI...", 
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information);
        }


    }
}
