#include <winsock2.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#define snew(x)	((x*)malloc(sizeof(x)))

typedef struct {
	const char *protocol;
	const char *server;
	const char *uri;
	unsigned short int port;

	char *_buf;
} url;

void url_destroy(url *u) {
	free(u->_buf);
	free(u);
}

url *url_parse(const char *s) {
	char *t, *ps;
	url *r = snew(url);
	r->_buf = (char*)malloc(strlen(s) + 4);
	strcpy(r->_buf, s);
	r->protocol = r->_buf;
	t = strstr(r->_buf, "://");
	if (!t) {
		r->protocol = "http";
		t = r->_buf;
	} else {
		*t = 0;
		t += 3;
	}
	r->server = t;
	ps = strchr(t, '/');
	if (!ps) {
		strcat(r->_buf, "/");
		ps = r->_buf + strlen(r->_buf) - 1;
	}
	t = strchr(t, ':');
	r->port = 80;
	if (t && t < ps) {
		int p;
		*t = 0;
		sscanf(t + 1, "%d", &p);
		r->port = p;
	}
	memmove(ps + 1, ps, strlen(ps) + 1);
	*(ps++) = 0;
	r->uri = ps;

	return r;

err:
	url_destroy(r);
	return NULL;
}

char *multiconcat(const char *s[], unsigned int *sz) {
	const char **t;
	unsigned int sl = 0;
	char *r;

	for (t = s; *t; ++t)
		sl += strlen(*t);

	r = (char*)malloc(sl + 1);
	*r = 0;
	for (t = s; *t; ++t)
		strcat(r, *t);

	if (sz) *sz = sl;
	return r;
}

int http_download(url *u) {
	struct hostent *he;
	struct sockaddr_in pin = {0};
	SOCKET s;
	char *request, *response;
	char rbuf[1024];
	int lr, rp;
	unsigned int rqsize;

	if (!(he = gethostbyname(u->server))) {
		fprintf(stderr, "Server nije nadjen!\n");
		return 1;
	}

	pin.sin_family = AF_INET;
	pin.sin_addr.s_addr = ((struct in_addr *)(he->h_addr))->s_addr;
	pin.sin_port = htons(u->port);
	
	/* grab an Internet domain socket */
	if ((s = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		fprintf(stderr, "Ne mogu stvoriti socket!\n");
		return 1;
	}
	
	/* connect to PORT on HOST */
	if (connect(s, (struct sockaddr *)&pin, sizeof(pin)) == -1) {
		fprintf(stderr, "Ne mogu se spojiti!\n");
		return 1;
	}


	{
		const char *mcsl[] = {
			"GET ",			// GET <uri> HTTP/1.0
			u->uri,
			" HTTP/1.0\r\n",
			"Host: ",		// Host: <server>
			u->server,
			"\r\n",
			"\r\n",			// gotovi headeri
			NULL
		};
		request = multiconcat(mcsl, &rqsize);
	}

	/* send a message to the server PORT on machine HOST */
	if (send(s, request, rqsize, 0) == -1) {
		fprintf(stderr, "Can't send!\n");
		return 1;
	}

	free(request);

	response = (char*)malloc(1);
	rp = 0;
	while ((lr = recv(s, rbuf, sizeof(rbuf), 0)) > 0) {
		response = (char*)realloc(response, rp + lr + 1);
		memcpy(response + rp, rbuf, lr);
		rp += lr;
	}
	//response[rp] = 0;

	write(1, response, rp);
	free(response);

	return 0;
}

int main(int argn, const char *argv[]) {
	url *u;
	int retval = 0;

	if (argn != 2) {
		fprintf(stderr, "Usage: %s url\n", argv[0]);
		return 1;
	}

	{
		WORD wVersionRequested = MAKEWORD(2, 2);
		WSADATA wsadata;
		if (WSAStartup(wVersionRequested, &wsadata)!= 0)
			return 2;
	}

	u = url_parse(argv[1]);
	if (!u) {
		fprintf(stderr, "Invalid url!\n");
		retval = 3;
		goto kraj;
	}

	if (strcmp(u->protocol, "http")) {
		fprintf(stderr, "Nepodrzan protokol!\n");
		retval = 3;
		goto kraj2;
	}

	retval = http_download(u) ? 4 : 0;

kraj2:
	url_destroy(u);
kraj:
	WSACleanup();
	return retval;
}
